/*************************************************************************************************
 * The command line interface of web crawler
 *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
 * This file is part of Hyper Estraier.
 * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License or any later version.  Hyper Estraier is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with Hyper
 * Estraier; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA.
 *************************************************************************************************/


#include "wavermod.h"
#include "myimage.dat"


#define NUMBUFSIZ      32                /* size of a buffer for a number */
#define URIBUFSIZ      8192              /* size of a buffer for an URI */


/* global variables */
const char *g_progname;                  /* program name */
int g_sigterm = FALSE;                   /* flag for termination signal */


/* function prototypes */
int main(int argc, char **argv);
static void usage(void);
static void printferror(const char *format, ...);
static void printfinfo(const char *format, ...);
static int rununittest(int argc, char **argv);
static int procunittest(const char *rootdir);


/* main routine */
int main(int argc, char **argv){
  const char *tmp;
  int rv;
  if((tmp = getenv("ESTDBGFD")) != NULL) dpdbgfd = atoi(tmp);
  cbstdiobin();
  g_progname = argv[0];
  g_sigterm = FALSE;
  if(argc < 2) usage();
  rv = 0;
  if(!strcmp(argv[1], "unittest")){
    rv = rununittest(argc, argv);
  } else {
    usage();
  }
  return rv;
}


/* print the usage and exit */
static void usage(void){
  fprintf(stderr, "%s: command line interface of web crawler\n", g_progname);
  fprintf(stderr, "\n");
  fprintf(stderr, "usage:\n");
  fprintf(stderr, "  %s unittest rootdir\n", g_progname);
  fprintf(stderr, "\n");
  exit(1);
}


/* print formatted error string and flush the buffer */
static void printferror(const char *format, ...){
  va_list ap;
  va_start(ap, format);
  fprintf(stderr, "%s: ERROR: ", g_progname);
  vfprintf(stderr, format, ap);
  fputc('\n', stderr);
  fflush(stderr);
  va_end(ap);
}


/* print formatted information string and flush the buffer */
static void printfinfo(const char *format, ...){
  va_list ap;
  va_start(ap, format);
  printf("%s: INFO: ", g_progname);
  vprintf(format, ap);
  putchar('\n');
  fflush(stdout);
  va_end(ap);
}


/* parse arguments of the unittest command */
static int rununittest(int argc, char **argv){
  char *rootdir;
  int i, rv;
  rootdir = NULL;
  for(i = 2; i < argc; i++){
    if(!rootdir && argv[i][0] == '-'){
      usage();
    } else if(!rootdir){
      rootdir = argv[i];
    } else {
      usage();
    }
  }
  if(!rootdir) usage();
  rv = procunittest(rootdir);
  return rv;
}


/* perform the unittest command */
static int procunittest(const char *rootdir){
  QUEUE *queue;
  char uri[URIBUFSIZ], *vbuf;
  int i, err;
  unlink(rootdir);
  if(!(queue = queue_open(rootdir))){
    printferror("%d: could not open", rootdir);
    return 1;
  }
  err = FALSE;
  printfinfo("checking priority queue");
  for(i = 0; i < 100; i++){
    sprintf(uri, "http://%05d/%x/%x.html",
            i + 1, (int)(est_random() * 0x1000000), (int)(est_random() * 0x1000000));
    if(!queue_enqueue(queue, uri, est_random())){
      err = TRUE;
      break;
    }
    if(i % 10 == 0) queue_spread(queue, 1.0);
  }
  if(err){
    printferror("%s: enqueue failed", rootdir);
    queue_close(queue);
    return 1;
  }

  for(i = 0; (vbuf = queue_dequeue(queue)) != NULL; i++){

    printf("%s\n", vbuf);
    free(vbuf);
  }
  if(i != 100){
    printferror("%s: dequeue failed", rootdir);
    queue_close(queue);
    return 1;
  }



  if(!queue_close(queue)){
    printferror("%s: could not close", rootdir);
    return 1;
  }


  return 0;
}



/* END OF FILE */
